%READSEM   Read SEM tiff file format from FEI 
%  pixel size is returned in [m]
%
% SYNOPSIS:
%  [image_out,ps] = readsem(filename, option)
%
% PARAMETERS:
%  filename: string with name of file
%  option:   'with bar', 'without bar','header info'
%
% DEFAULTS:
%  option: 'without bar'
%
% NOTE:
%  Retreval of pixel size only works if tiffinfo.exe is on the system path 

% (C) Copyright 2007               FEI Electron Optics - Building AAEp 
%     All rights reserved          PO Box 80066
%                                  5600 KA Eindhoven 
%                                  The Netherlands
% Bernd Rieger, Feb 2007

function varargout = readsem(varargin)
d = struct('menu','FEI',...
  'display','Read SEM tiff file',...
  'inparams',struct('name',       {'filename','sp'},...
                    'description',{'Name of the file to open'},...
                    'type',       {'infile','option'},...
                    'dim_check',  {0,[]},...
                    'range_check',{'*.tif',{'With bar','Without bar','Header info'}},...
                    'required',   {1,0},...
                    'default',    {'bla.tif','Without bar'}...
                   ),...
  'outparams',struct('name',{'out','ps',''},...
                     'description',{'Output image','Pixel size [m]','Header info'},...
                     'type',{'image','array','string'}...
                     )...
 );
if nargin == 1
   s = varargin{1};
   if ischar(s) & strcmp(s,'DIP_GetParamList')
      varargout{1} = d;
      return
   end
end
try
   [fn,sp] = getparams(d,varargin{:});
catch
   if ~isempty(paramerror)
      error(paramerror)
   else
      error(firsterr)
   end
end
varargout{1} = readgrayim(fn);

[s,w]=system(['tiffinfo ' fn]);
if s |  findstr(w,'Cannot open')
   error(['Could not use tiffinfo to open file ' fn ', error: ' w]);
end

   i = findstr(w,'ResolutionX');
   if isempty(i)
      warning('Could not determine the resolution.');
      varargout{2} =[];
      return;
   end
   sz(1) = get_number(w,findstr(w,'ResolutionX')+length('ResolutionX')+1);
   sz(2) = get_number(w,findstr(w,'ResolutionY')+length('ResolutionY')+1);
   %fprintf(' Found image size %d %d\n',sz)
   if strcmp(sp,'Without bar')
      out = varargout{1};
      out = out(0:sz(1)-1,0:sz(2)-1); %crop away the information bar
      varargout{1} = out;
   end
   i=findstr(w,'PixelWidth')+length('PixelWidth')+1;
   ps(1) = get_number(w,i(1));
   i=findstr(w,'PixelHeight')+length('PixelHeight')+1;
   ps(2) = get_number(w,i(2));
   %fprintf(' Found pixel size %e %e\n',ps)
varargout{2} =ps;

if strcmp(sp,'Header info')
   i = findstr(w,'System');
   varargout{3} = w(i:end);
end


function out= get_number(in,ind)
%ok maybe not the smartest way, but quick hack
out = zeros(length(ind),1);
for ii=1:length(ind)
   ok = 1;
   jj=0;
   while ok
      jj=jj+1;
      [tn,ok] = str2num(in(ind(ii)+jj));
      if ~ok %check if '.' or 'e' for float
         if findstr(in(ind(ii)+jj),'.eE-+') 
            ok=1;
         end
      end
   end
   in(ind(ii):ind(ii)+jj-1);
   out(ii) = str2num(in(ind(ii):ind(ii)+jj-1));
end
